//
//  SSCBaseViewController.m
//  SimplySelfCare
//
//  Created by Michael Place on 8/29/12.
//  Copyright (c) 2012 scrapsfromthetable. All rights reserved.
//

#import "SSCBaseViewController.h"

@interface SSCBaseViewController ()

@end

@implementation SSCBaseViewController
@synthesize currentUser = _currentUser;
@synthesize isLoggedIn;
@synthesize userManagementViewController = _userManagementViewController;
@synthesize ailmentViewController = _ailmentViewController;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void) setupSwipeGestureRecognizer {
    //UISwipeGestureRecognizer *swipeLeftGesture = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(swipedScreen:)];
    //swipeLeftGesture.direction = (UISwipeGestureRecognizerDirectionLeft);
    //UISwipeGestureRecognizer *swipeRightGesture = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(swipedScreen:)];
    //swipeRightGesture.direction = (UISwipeGestureRecognizerDirectionRight);
    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hideKeyboard)];
    tapGesture.cancelsTouchesInView = NO;
    //[self.view addGestureRecognizer:swipeLeftGesture];
    //[self.view addGestureRecognizer:swipeRightGesture];
    [self.contentScrollView addGestureRecognizer:tapGesture];
}

- (void)viewDidLoad
{
    if (!isLoggedIn) {
        [self presentLoginForm];
    }
    
    [self.contentScrollView setContentSize:CGSizeMake(self.view.frame.size.width, 640)];
    isInMedicineCabinet = NO;
    isEditMode = NO;
    [self setupSwipeGestureRecognizer];
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
}

-(void)viewDidAppear:(BOOL)animated {
    if (!isLoggedIn) {
        [self presentLoginForm];
    }
    isInMedicineCabinet = NO;
    isEditMode = NO;
}

-(void)presentLoginForm {
    self.splashVC = [[SSCSplashViewController alloc] initWithNibName:@"SSCSplashViewController" bundle:[NSBundle mainBundle]];
    self.splashVC.delegate = self;
    [self.navigationController presentModalViewController:self.splashVC animated:NO];
}

- (void)splashViewDidSuccessfullyLoginWithUser:(SSCUser*)user {
    NSLog(@"In Delegate Method");
    
    [self setCurrentUser:user];
    [self.navigationController dismissModalViewControllerAnimated:YES];
    [self setIsLoggedIn:YES];
    [self populateProfileLabelsWithUser:user];
    [[self userFunctionsBarButtonItem] setTitle:user.username];
    [SSCUser fetchUser:user WithPredicateString:[NSPredicate predicateWithFormat:@"username == %@",user.username]];
}

-(void)populateProfileLabelsWithUser:(SSCUser*)user {
    self.usernameProfileLabel.text = user.username;
    self.genderProfileLabel.text = ([user.gender isEqualToNumber:[NSNumber numberWithInt:Male]] ? @"Male":@"Female");
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"MM/dd/yyyy"];
    NSString *dateOfBirth = [dateFormatter stringFromDate:user.dob];
    self.ageProfileLabel.text = dateOfBirth;
    NSString *currentMedicationsForUser = [[NSString alloc] init];
    if (nil != user.currentMedications) {
        for (NSString *med in user.currentMedications) {
            currentMedicationsForUser = [currentMedicationsForUser stringByAppendingFormat:@"%@, ", med];
        }
        currentMedicationsForUser = [currentMedicationsForUser substringToIndex:[currentMedicationsForUser length] - 2];
        self.currentMedicationProfileLabel.text = currentMedicationsForUser;
    }
    else {
        self.currentMedicationProfileLabel.text = @"None";
    }
    NSString *currentConditionsForUser = [[NSString alloc] init];
    if (nil != user.currentConditions) {
        for (NSString *cond in user.currentConditions) {
            currentConditionsForUser = [currentConditionsForUser stringByAppendingFormat:@"%@, ", cond];
        }
        currentConditionsForUser = [currentConditionsForUser substringToIndex:[currentConditionsForUser length] - 2];
        self.currentConditionProfileLabel.text = currentConditionsForUser;
    }
    else {
        self.currentConditionProfileLabel.text = @"None";
    }
    self.emailProfileLabel.text = user.email;
    
    
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)swipedScreen:(UISwipeGestureRecognizer*)gesture
{
    if (gesture.direction == UISwipeGestureRecognizerDirectionRight && isInMedicineCabinet) {
        [self animateMedicineCabinet];
    }
    NSLog(@"Gesture Recognized");
    if (gesture.direction == UISwipeGestureRecognizerDirectionLeft && !isInMedicineCabinet) {
        [self animateMedicineCabinet];
    }    
}

- (void)getServerSymptomsForBodyRegion:(NSString*)bodyRegion withCompletion:(void (^)())completion {
    NSDictionary *json = [NSDictionary dictionaryWithObjects:@[bodyRegion] forKeys:@[@"BodyRegion"]];
    NSError *error;
    NSMutableDictionary *jsonMutableDictionary = [[NSMutableDictionary alloc] initWithDictionary:@{}];
    NSData* jsonCredData = [NSJSONSerialization dataWithJSONObject:json
                                                           options:NSJSONWritingPrettyPrinted error:&error];
    if (nil == error) {
        NSString* stringData = [[NSString alloc] initWithData:jsonCredData
                                                     encoding:NSUTF8StringEncoding];
        jsonMutableDictionary = [[NSMutableDictionary alloc]initWithDictionary:@{@"BodyRegion" : stringData}];
    }
    else {
        NSLog(@"Error creating JSON data object");
    }
    
    
    MKNetworkOperation *op = [[[SSCMKNetworkSingleton sharedInstance] nwe] operationWithPath:kSSCSymptomsForRegion
                                             params:jsonMutableDictionary
                                         httpMethod:@"POST"];
    [op onCompletion:^(MKNetworkOperation *operation) {
        
        NSDictionary *jsonResponseDictionary = [operation responseJSON];
        if (nil == jsonResponseDictionary) {
            NSLog(@"RESPONSE FROM SERVER IS NIL!!");
        }
     else {
         NSArray *symptoms = [[jsonResponseDictionary objectForKey:@"symptoms"]componentsSeparatedByString:@","];
         NSLog(@"%@", [symptoms description]);
         
         dispatch_async(dispatch_get_main_queue(), ^{
             completion();
             [self presentAilmentViewControllerWithSymptoms:symptoms andCurrentUser:self.currentUser andBodyRegion:bodyRegion];
         });
     }
    }onError:^(NSError *error) {
        
        DLog(@"%@", error);
        UIAlertView *failedToConnectToServer = [[UIAlertView alloc] initWithTitle:@"Failed to connect to the Simply Selfcare Server" message:@"We apologize, please try again soon." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [failedToConnectToServer show];
    }];
    [[[SSCMKNetworkSingleton sharedInstance] nwe] enqueueOperation:op];
}

- (void)startActivityIndicator {
    if (nil == self.aiv) {
        self.aiv = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
    }
    [self.aiv setCenter:self.view.center];
    [self.view addSubview:self.aiv];
    [self.aiv startAnimating];
}

- (IBAction)didClickHeadButton:(id)sender {
    [self startActivityIndicator];
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self getServerSymptomsForBodyRegion:@"head" withCompletion:^() {
            [self.aiv stopAnimating];
        }];
    });
    
}

- (IBAction)didClickChestButton:(id)sender {
    [self startActivityIndicator];
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self getServerSymptomsForBodyRegion:@"chest" withCompletion:^() {
            [self.aiv stopAnimating];
        }];
    });
}

- (IBAction)didClickStomachButton:(id)sender {
    [self startActivityIndicator];
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self getServerSymptomsForBodyRegion:@"stomach" withCompletion:^() {
            [self.aiv stopAnimating];
        }];
    });
}

- (IBAction)didClickLegsButton:(id)sender {
    [self startActivityIndicator];
    // Not yet utilized
}

- (IBAction)didClickSaveProfileStatsButton:(id)sender {
    [self saveEditChangesToProfileStatsAndCurrentUser];
    [self toggleEditMode:YES animated:YES];
}

- (IBAction)animateMedicineCabinet {
    if (isInMedicineCabinet == NO) {
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationCurve:UIViewAnimationCurveLinear];
        [UIView setAnimationDuration:.5];
        [self.profileView setFrame:CGRectMake(self.profileView.frame.origin.x - 300, self.profileView.frame.origin.y, self.profileView.frame.size.width, _profileView.frame.size.height)];
        [UIView commitAnimations];
        isInMedicineCabinet = YES;
    }
    else {
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationCurve:UIViewAnimationCurveLinear];
        [UIView setAnimationDuration:.5];
        [self.profileView setFrame:CGRectMake(self.profileView.frame.origin.x + 300, self.profileView.frame.origin.y, self.profileView.frame.size.width, self.profileView.frame.size.height)];
        [UIView commitAnimations];
        isInMedicineCabinet = NO;
    }
    NSLog(@"Animating Medicine Cabinet");
    
    
}

- (IBAction)displayUserFamily:(id)sender {
    NSLog(@"Displaying User Family");
    if (nil == self.userManagementViewController) {
        self.userManagementViewController = [[SSCUserManagementViewController alloc] init];
    }
    self.userManagementViewController.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
    self.userManagementViewController.delegate = self;
    self.userManagementViewController.currentUser = self.currentUser;
    [self presentModalViewController:self.userManagementViewController
                                                   animated:YES];
    
}

- (IBAction)didClickEditProfileStatsButton:(UIButton *)sender {
    [self toggleEditMode:(isEditMode ? YES : NO) animated:YES];
}

- (void)toggleEditMode:(BOOL)yn animated:(BOOL)animated{
    NSArray *editModeViews = [self getEditModeTextFields];
    NSArray *profileStatViews = [self getProfileStatViews];
    //NSArray *editModeAddButtons = @[self.addCurrentConditionButton, self.addCurrentMedicationButton];
    if (NO == yn){
        if (YES == animated) {
            [UIView beginAnimations:nil context:nil];
            [UIView setAnimationCurve:UIViewAnimationCurveEaseIn];
            [UIView setAnimationDuration:.5];
            [self setAlpha:1 ForViewArray:editModeViews];
            //[self setAlpha:1 ForViewArray:editModeAddButtons];
            [self setAlpha:0 ForViewArray:profileStatViews];
            [self.saveProfileStatsButton setAlpha:1];
            [UIView commitAnimations];
            [self.editProfileStatsButton setTitle:@"cancel" forState:UIControlStateNormal];
            [self populateEditFieldsWithPlaceholders];
            isEditMode = YES;
        }
        else {
            [self setAlpha:1 ForViewArray:editModeViews];
            //[self setAlpha:1 ForViewArray:editModeAddButtons];
            [self setAlpha:0 ForViewArray:profileStatViews];
            [self.saveProfileStatsButton setAlpha:1];
            [self.editProfileStatsButton setTitle:@"cancel" forState:UIControlStateNormal];
            [self populateEditFieldsWithPlaceholders];
            isEditMode = YES;
        }
    }
    else {
        if (YES == animated) {
            [UIView beginAnimations:nil context:nil];
            [UIView setAnimationCurve:UIViewAnimationCurveEaseIn];
            [UIView setAnimationDuration:.5];
            [self setAlpha:0 ForViewArray:editModeViews];
            //[self setAlpha:0 ForViewArray:editModeAddButtons];
            [self setAlpha:1 ForViewArray:profileStatViews];
            [self.saveProfileStatsButton setAlpha:0];
            [UIView commitAnimations];
            [self.editProfileStatsButton setTitle:@"edit" forState:UIControlStateNormal];
            isEditMode = NO;
        }
        else {
            [self setAlpha:0 ForViewArray:editModeViews];
            //[self setAlpha:0 ForViewArray:editModeAddButtons];
            [self setAlpha:1 ForViewArray:profileStatViews];
            [self.saveProfileStatsButton setAlpha:0];
            [self.editProfileStatsButton setTitle:@"edit" forState:UIControlStateNormal];
            isEditMode = NO;
        }
    }
}

- (void)populateEditFieldsWithPlaceholders {
    self.modifyEmailTextView.text = self.emailProfileLabel.text;
    self.modifyAgeTextView.text = self.ageProfileLabel.text;
    self.modifyGenderTextView.text = self.genderProfileLabel.text;
    self.modifyCurrentConditionsTextView.text = self.currentConditionProfileLabel.text;
    self.modifyCurrentMedicationsTextView.text = self.currentMedicationProfileLabel.text;
}

- (void)saveEditChangesToProfileStatsAndCurrentUser {
    self.emailProfileLabel.text = self.modifyEmailTextView.text;
    self.ageProfileLabel.text = self.modifyAgeTextView.text;
    self.genderProfileLabel.text = self.modifyGenderTextView.text;
    self.currentConditionProfileLabel.text = self.modifyCurrentConditionsTextView.text;
    self.currentMedicationProfileLabel.text = self.modifyCurrentMedicationsTextView.text;
    
    // Update Local User Variable
    self.currentUser.email = self.emailProfileLabel.text;
    self.currentUser.gender = [SSCUser genderNumberFromString:self.genderProfileLabel.text];
    self.currentUser.dob = [SSCUser birthDateFromString:self.ageProfileLabel.text];
    self.currentUser.currentConditions = [SSCUser arrayFromStringOfConditionsOrMedications:self.currentConditionProfileLabel.text];
    self.currentUser.currentMedications = [SSCUser arrayFromStringOfConditionsOrMedications:self.currentMedicationProfileLabel.text];
    
    // Add to Local User Store, 
    //[SSCUser manageLocalUserStoreWithUser:self.currentUser];
    [SSCUser fetchUser:self.currentUser WithPredicateString:[NSPredicate predicateWithFormat:@"username == %@",self.currentUser.username]];
    [SSCUser applyProfileUserChangesRemotelyWithUser:self.currentUser];
}

- (void)presentAilmentViewControllerWithSymptoms:(NSArray*)s andCurrentUser:(SSCUser*)currUser andBodyRegion:(NSString*)bodyRegion{
    if (self.ailmentViewController != nil) {
        self.ailmentTableView = nil;
    }
    self.ailmentViewController = [[SSCAilmentViewController alloc] initWithNibName:@"SSCAilmentViewController" bundle:nil];
    self.ailmentViewController.delegate = self;
    self.ailmentViewController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
    self.ailmentViewController.symptoms = s;
    self.ailmentViewController.currentUser = currUser;
    self.ailmentViewController.bodyRegion = bodyRegion;
    [self.navigationController pushViewController:self.ailmentViewController animated:YES];
//    [self presentModalViewController:self.ailmentViewController animated:YES];
}

- (void)invalidateUserForUserWithUsername:(NSString*)username{
    self.currentUser = [SSCUser fetchUserWithUsername:username];
    [self populateProfileLabelsWithUser:self.currentUser];
}

- (void)diagnosisWasCancelled {
    NSLog(@"In diagnosisWasCancelled method");
    [self.navigationController popViewControllerAnimated:YES];
    self.ailmentViewController = nil;
}

-(void)didRequestLogout {
    [self setIsLoggedIn:NO];
    [self dismissModalViewControllerAnimated:NO];
    
}

-(void)didRequestChangeUser:(NSString*)username {
    [self invalidateUserForUserWithUsername:username];
    [self dismissModalViewControllerAnimated:YES];
}

-(void)hideKeyboard
{
    [[self view] endEditing:TRUE];
}

- (NSArray*)getEditModeTextFields {
    NSMutableArray *textFields = [[NSMutableArray alloc] init];
    for (UIView* v in self.contentScrollView.subviews) {
        if ([v isMemberOfClass:[UITextField class]]) {
            [textFields addObject:v];
        }
    }
    NSArray *tfArray = [NSArray arrayWithArray:textFields];
    return tfArray;
}

- (NSArray*)getProfileStatViews {
    NSArray *profileStatViews = @[_genderProfileLabel, _ageProfileLabel, _currentConditionProfileLabel, _currentMedicationProfileLabel, _emailProfileLabel];
    return profileStatViews;
}

- (void)setAlpha:(int)a ForViewArray:(NSArray*)viewArray {
    for (UIView* view in viewArray) {
        [view setAlpha:a];
    }
}

- (void)viewDidUnload
{
    [self setUsernameProfileLabel:nil];
    [self setGenderProfileLabel:nil];
    [self setAgeProfileLabel:nil];
    [self setCurrentMedicationProfileLabel:nil];
    [self setCurrentConditionProfileLabel:nil];
    [self setRevealMedicineCabinetButton:nil];
    [self setBackgroundGestureRecogizerView:nil];
    [self setUserFunctionsBarButtonItem:nil];
    [self setModifyEmailTextView:nil];
    [self setModifyGenderTextView:nil];
    [self setModifyAgeTextView:nil];
    [self setModifyCurrentConditionsTextView:nil];
    [self setModifyCurrentMedicationsTextView:nil];
    [self setAddCurrentConditionButton:nil];
    [self setAddCurrentMedicationButton:nil];
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

@end
