//
//  SSCAilmentViewController.m
//  SimplySelfCare
//
//  Created by Michael Place on 8/30/12.
//  Copyright (c) 2012 scrapsfromthetable. All rights reserved.
//

#import "SSCAilmentViewController.h"
#import <QuartzCore/QuartzCore.h>
#import "SSCMKNetworkSingleton.h"
#import "SSCViableMedViewController.h"

@interface SSCAilmentViewController ()

@end

@implementation SSCAilmentViewController
@synthesize delegate, symptomTableView;
@synthesize selectedSymptoms = _selectedSymptoms;
@synthesize symptoms = _symptoms;
@synthesize currentUser = _currentUser;
@synthesize bodyRegion = _bodyRegion;
@synthesize aiv = _aiv;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    [self.symptomTableView.layer setBorderWidth:2.0];
    [self.symptomTableView.layer setBorderColor:[UIColor blackColor].CGColor];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)makeViableDrugRequestWithSymptoms:(NSArray*)symptoms withCompletion:(void (^)())completion{
    NSString *commaDelimSymptoms = [symptoms componentsJoinedByString:@","];
    NSString *commaDelimCurrentMeds = [[self.currentUser currentMedications] componentsJoinedByString:@","];
    NSString *commaDelimCurrentConds = [[self.currentUser currentConditions] componentsJoinedByString:@","];
    NSDictionary *json = [NSDictionary dictionaryWithObjects:@[commaDelimSymptoms, commaDelimCurrentMeds, commaDelimCurrentConds] forKeys:@[self.bodyRegion, @"CurrentMeds", @"CurrentConds"]];
    NSError *error;
    NSMutableDictionary *jsonMutableDictionary = [[NSMutableDictionary alloc] initWithDictionary:@{}];
    NSData* jsonCredData = [NSJSONSerialization dataWithJSONObject:json
                                                           options:NSJSONWritingPrettyPrinted error:&error];
    if (nil == error) {
        NSString* stringData = [[NSString alloc] initWithData:jsonCredData
                                                     encoding:NSUTF8StringEncoding];
        jsonMutableDictionary = [[NSMutableDictionary alloc]initWithDictionary:@{self.bodyRegion : stringData}];
    }
    else {
        NSLog(@"Error creating JSON data object");
    }
    
    
    MKNetworkOperation *op = [[[SSCMKNetworkSingleton sharedInstance] nwe] operationWithPath:kSSCGetViableMeds
                                                                                      params:jsonMutableDictionary
                                                                                  httpMethod:@"POST"];
    [op onCompletion:^(MKNetworkOperation *operation) {
        
        NSDictionary *jsonResponseDictionary = [operation responseJSON];
        if (nil == jsonResponseDictionary) {
            NSLog(@"RESPONSE FROM SERVER IS NIL!!");
        }
        else {
            NSArray *meds = [[jsonResponseDictionary objectForKey:@"ViableMeds"]componentsSeparatedByString:@","];
            NSLog(@"%@", [meds description]);
            
            dispatch_async(dispatch_get_main_queue(), ^{
                completion();
                SSCViableMedViewController *viableMedViewController = [[SSCViableMedViewController alloc] initWithNibName:@"SSCViableMedViewController" bundle:[NSBundle mainBundle]];
                [viableMedViewController setCurrentUser:self.currentUser];
                [viableMedViewController setViableMeds:meds];
                [self.navigationController pushViewController:viableMedViewController animated:YES];
            });
        }
    }onError:^(NSError *error) {
        
        DLog(@"%@", error);
        UIAlertView *failedToConnectToServer = [[UIAlertView alloc] initWithTitle:@"Failed to connect to the Simply Selfcare Server" message:@"We apologize, please try again soon." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [failedToConnectToServer show];
    }];
    [[[SSCMKNetworkSingleton sharedInstance] nwe] enqueueOperation:op];
}

- (IBAction)nextDiagnosisButtonClicked:(id)sender {
    NSMutableArray *selectedSymptoms = [[NSMutableArray alloc] init];
    for (int section = 0; section < [self.symptomTableView numberOfSections]; section++) {
        for (int row = 0; row < [self.symptomTableView numberOfRowsInSection:section]; row++) {
            NSIndexPath* cellPath = [NSIndexPath indexPathForRow:row inSection:section];
            UITableViewCell* cell = [self.symptomTableView cellForRowAtIndexPath:cellPath];
            if (cell.accessoryType == UITableViewCellAccessoryCheckmark) {
                [selectedSymptoms addObject:[self.symptoms objectAtIndex:row]];
            }
        }
    }
    [self startActivityIndicator];
    [self makeViableDrugRequestWithSymptoms:selectedSymptoms withCompletion:^{
        [self.aiv stopAnimating];
    }];
}

- (IBAction)backDiagnosisButtonClicked:(id)sender {
    NSLog(@"Back Button Was Clicked");
    if ([delegate respondsToSelector:@selector(diagnosisWasCancelled)]) {
        [delegate diagnosisWasCancelled];
    }
}

- (IBAction)cancelDiagnosisButtonClicked:(id)sender {
    NSLog(@"Cancel Button Was Clicked");
    if ([delegate respondsToSelector:@selector(diagnosisWasCancelled)]) {
        [delegate diagnosisWasCancelled];
    }
}

- (void)startActivityIndicator {
    if (nil == self.aiv) {
        self.aiv = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
    }
    [self.aiv setCenter:self.view.center];
    [self.view addSubview:self.aiv];
    [self.aiv startAnimating];
}

#pragma -TableViewDelegateMethods

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
    
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Cell"];
    }
    
    // Add elements to the cell
    [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
    cell.textLabel.text = [self.symptoms objectAtIndex:[indexPath row]];
    cell.textLabel.font = [UIFont fontWithName:@"System" size:14];
    return cell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.symptoms count];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if([tableView cellForRowAtIndexPath:indexPath].accessoryType == UITableViewCellAccessoryCheckmark){
        [tableView cellForRowAtIndexPath:indexPath].accessoryType = UITableViewCellAccessoryNone;
    }else{
        [tableView cellForRowAtIndexPath:indexPath].accessoryType = UITableViewCellAccessoryCheckmark;
    }
}

@end
