//
//  SSCAppDelegate.m
//  SimplySelfCare
//
//  Created by Michael Place on 8/24/12.
//  Copyright (c) 2012 scrapsfromthetable. All rights reserved.
//

#import "SSCAppDelegate.h"
#import "SSCDiagnosisStore.h"

@implementation SSCAppDelegate
@synthesize _mainWindow;

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // Override point for customization after application launch.
    baseViewController = [[SSCBaseViewController alloc] initWithNibName:@"SSCBaseViewController" bundle:[NSBundle mainBundle]];
    appNav = [[UINavigationController alloc] initWithRootViewController:baseViewController];
    [appNav setNavigationBarHidden:YES];
    [self._mainWindow setRootViewController:appNav];
    [self._mainWindow makeKeyAndVisible];
    
    ////////////////////// Core Data Manipulation ///////////////////////////
    NSString *storePath = [[[SSCDiagnosisStore sharedInstance] applicationDocumentsDirectory]
                           stringByAppendingPathComponent: @"pharm.sqlite"];
    
    // Put down default db if it doesn't already exist
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if (![fileManager fileExistsAtPath:storePath]) {
        NSString *defaultStorePath = [[NSBundle mainBundle]
                                      pathForResource:@"pharm" ofType:@"sqlite"];
        if (defaultStorePath) {
            [fileManager copyItemAtPath:defaultStorePath toPath:storePath error:NULL];
        }
    }
    
    NSManagedObjectContext *context = [[SSCDiagnosisStore sharedInstance] managedObjectContext];
    NSManagedObject *testDiagnosis = [NSEntityDescription
                                       insertNewObjectForEntityForName:@"Diagnosis"
                                       inManagedObjectContext:context];
    [testDiagnosis setValue:@"cough" forKey:@"symptoms"];
    [testDiagnosis setValue:@"allergy/cold" forKey:@"type"];
    [testDiagnosis setValue:0 forKey:@"ispregnant"];
    [testDiagnosis setValue:0 forKey:@"isproductive"];
    [testDiagnosis setValue:0 forKey:@"islactating"];
    [testDiagnosis setValue:0 forKey:@"gender"];
    [testDiagnosis setValue:[NSDate date] forKey:@"dob"];
    NSManagedObject *testOTCMed = [NSEntityDescription
                                          insertNewObjectForEntityForName:@"OTCMed"
                                          inManagedObjectContext:context];
    [testOTCMed setValue:@"ibuprofen" forKey:@"name"];
    [testDiagnosis setValue:[NSSet setWithArray:@[testOTCMed]] forKey:@"otcmeds"];
    [testOTCMed setValue:testDiagnosis forKey:@"diagnosis"];
    NSError *error;
    if (![context save:&error]) {
        NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
    }
    
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

@end
