//
//  SSCSplashViewController.m
//  SimplySelfCare
//
//  Created by Michael Place on 9/29/12.
//  Copyright (c) 2012 scrapsfromthetable. All rights reserved.
//

#import "SSCSplashViewController.h"

@interface SSCSplashViewController ()
- (void)addRemoteDatabaseEntryForUser:(SSCUser *)user;
- (void)signInToRemoteDatabaseWithUser:(NSDictionary*)credentials;
- (NSDate*)birthDateFromLoginFields;
@end

@implementation SSCSplashViewController
@synthesize delegate, userName, password, signIn, createNew, email, gender, dob_month, dob_day, dob_year, dob_slash1, dob_slash2, currentUser, loginFormScrollView;
@synthesize switchUserFlag = _switchUserFlag;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //initialize view state
    isCreatingNewUser = NO;
    isSigningIn = YES;
    loginVerified = NO;
    [self.password setSecureTextEntry:YES];
    [self.userName becomeFirstResponder];
    
    // register for keyboard notifications
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillShow:)
                                                 name:UIKeyboardWillShowNotification
                                               object:self.view.window];
    // register for keyboard notifications
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification
                                               object:self.view.window];
    keyboardIsShown = NO;
    //make contentSize bigger than your scrollSize (you will need to figure out for your own use case)
    CGSize scrollContentSize = CGSizeMake(320, 400);
    self.loginFormScrollView.contentSize = scrollContentSize;
    
    UITapGestureRecognizer *tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hideKeyboard)];
    tapGesture.cancelsTouchesInView = NO;
    [loginFormScrollView addGestureRecognizer:tapGesture];
    
}

-(void)viewDidAppear:(BOOL)animated {
    if (self.switchUserFlag == YES) {
        self.userName.text = currentUser.username;
        self.password.text = currentUser.password;
    }
}

-(IBAction)loginButtonsWereTapped:(id)sender {
    if (sender == signIn) {
        if (isSigningIn) {
            [self handleUserDataForServerAuthorization];
        }
        else {
            //Cancel button pressed, re-enter sign-in mode
            isSigningIn = YES;
            isCreatingNewUser = NO;
            [self animateLoginFormButtons];
        }
    }
    else {
        

        if (sender == createNew){
            if (isSigningIn) {
                //changing to create new user mode
                isSigningIn = NO;
                isCreatingNewUser = YES;
                [self animateLoginFormButtons];
            }
        else {
            //Start the process of adding new user to remote database
            if ([self validateEnteredLoginData]) {
                [self handleUserDataForServerAuthorization];
            }
            else {
                UIAlertView *invalidDataAlertView = [[UIAlertView alloc] initWithTitle:@"Information Entered Incorrectly" message:@"Please make sure that all fields are filled out." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
                [invalidDataAlertView show];
            }
        }
    
        }
    }
    [password resignFirstResponder];
}

- (BOOL)validateEnteredLoginData {
    NSArray *fields = [[NSArray alloc] initWithObjects:userName, password, email, dob_month, dob_day, dob_year, nil];
    BOOL dataIsLegit = YES;
    for (UITextField* tv in fields) {
        if ([tv.text isEqualToString:@""]) {
            dataIsLegit = NO;
        }
    }
    return dataIsLegit;
}

-(void)handleUserDataForServerAuthorization {
    
    if (isCreatingNewUser) {
        currentUser = [[SSCUser alloc] initUserWith:self.userName.text withPassword:self.password.text withEmail:self.email.text withGender:[NSNumber numberWithInt:[self.gender selectedSegmentIndex]] withDOB:self.birthDateFromLoginFields withIsPregnant:[NSNumber numberWithBool:NO] withIsLactating:[NSNumber numberWithBool:NO] withCurrentMedications:@[] withCurrentConditions:@[] withMedicineCabinet:@[]];
        [self addRemoteDatabaseEntryForUser:currentUser];
    }
    else {
        NSLog(@"Username: %@", userName.text);
        NSLog(@"Password: %@", password.text);
        NSDictionary *signInDict = [[NSDictionary alloc] initWithObjects:@[self.userName.text,self.password.text] forKeys:@[@"username", @"password"]];
        [self signInToRemoteDatabaseWithUser:signInDict];
    }
}

-(void)launchHomeView {
    // if anyone is listening
    if([delegate respondsToSelector:@selector(splashViewDidSuccessfullyLoginWithUser:)])
    {
        // login successful, launch homeview
        [delegate splashViewDidSuccessfullyLoginWithUser:currentUser];
    }
    
}



- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIKeyboardWillShowNotification
                                                  object:nil];
    // unregister for keyboard notifications while not visible.
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIKeyboardWillHideNotification
                                                  object:nil];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark Server Authentication Methods

- (void)addRemoteDatabaseEntryForUser:(SSCUser *)user {
    NSDictionary *userDict = [user dictionaryObjectForUser:user];
    NSData *jsonUserData = [user jsonRepresentationForUserDictionary:userDict];
    NSString* stringData = [[NSString alloc] initWithData:jsonUserData
                                                 encoding:NSUTF8StringEncoding];
    NSMutableDictionary *jsonMutableDictionary = [[NSMutableDictionary alloc]initWithDictionary:@{@"adduser" : stringData}];
    [self queryWithPostData:jsonMutableDictionary andWithURL:kSSCAuth];
    
}

- (void)signInToRemoteDatabaseWithUser:(NSDictionary*)credentials {
    NSError *error;
    NSData* jsonCredData = [NSJSONSerialization dataWithJSONObject:credentials
                                                           options:NSJSONWritingPrettyPrinted error:&error];
    if (nil == error) {
        NSString* stringData = [[NSString alloc] initWithData:jsonCredData
                                                     encoding:NSUTF8StringEncoding];
        NSMutableDictionary *jsonMutableDictionary = [[NSMutableDictionary alloc]initWithDictionary:@{@"signin" : stringData}];
        [self queryWithPostData:jsonMutableDictionary andWithURL:kSSCAuth];
    }
    else {
        NSLog(@"Error creating JSON data object");
    }
    
}

- (void)queryWithPostData:(NSMutableDictionary*)jsonDataDictionary andWithURL:(NSString*)pharmacyURL {
    MKNetworkOperation *op = [[[SSCMKNetworkSingleton sharedInstance] nwe] operationWithPath:pharmacyURL
                                             params:jsonDataDictionary
                                          httpMethod:@"POST"];
    [op onCompletion:^(MKNetworkOperation *operation) {
        
        NSDictionary *jsonResponseDictionary = [operation responseJSON];
        if (nil == jsonResponseDictionary) {
            NSLog(@"RESPONSE FROM SERVER IS NIL!!");
        }
        if ([[jsonResponseDictionary allKeys] containsObject:@"requestStatus"] && [jsonResponseDictionary objectForKey:@"requestStatus"] == (id)[NSNull null]) {
            UIAlertView *wrongPassword = [[UIAlertView alloc] initWithTitle:@"Wrong Username or Password" message:@"We were unable to log you in with the username or password provided. Please try again." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [wrongPassword show];
        }
        else {
            if ([[jsonResponseDictionary allKeys] containsObject:@"requestStatus"] && ![[jsonResponseDictionary objectForKey:@"requestStatus"] isEqualToNumber:[NSNumber numberWithInt:0]]) {
                loginVerified = NO;
                if (isSigningIn) {
                    UIAlertView *failedToLogUserIn = [[UIAlertView alloc] initWithTitle:@"Failed to Log in." message:@"We were unable to log you in at the moment. Please try again." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                    [failedToLogUserIn show];
                }
                else {
                    UIAlertView *usernameAlreadyExistsInDatabase = [[UIAlertView alloc] initWithTitle:@"Failed to add user because username is taken" message:@"We were unable to create a user with the information provided. Please try again." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
                    [usernameAlreadyExistsInDatabase show];
                }
            }
            else {
                if (isSigningIn) {
                    currentUser = [[SSCUser alloc] init];
                    [self.currentUser setUsername:[jsonResponseDictionary objectForKey:@"username"]];
                    [self.currentUser setPassword:[jsonResponseDictionary objectForKey:@"password"]];
                    [self.currentUser setEmail:[jsonResponseDictionary objectForKey:@"email"]];
                    [self.currentUser setDob:[SSCUser birthDateFromString:[jsonResponseDictionary objectForKey:@"dob"]]];
                    [self.currentUser setGender:[jsonResponseDictionary objectForKey:@"gender"]];
                    NSArray *tmpCurrentCondtions = [[jsonResponseDictionary objectForKey:@"currentconditions"] componentsSeparatedByString: @","];
                    [self.currentUser setCurrentConditions:tmpCurrentCondtions];
                    NSArray *tmpCurrentMedications = [[jsonResponseDictionary objectForKey:@"currentmedications"] componentsSeparatedByString: @","];
                    [self.currentUser setCurrentMedications:tmpCurrentMedications];
                    [self.currentUser setIsPreg:[jsonResponseDictionary objectForKey:@"ispregnant"]];
                    [self.currentUser setIsLac:[jsonResponseDictionary objectForKey:@"islactating"]];
                }
                else {
                    //User has been added so current user is good they way it was initialized
                    currentUser = currentUser;
                }
                [SSCUser fetchUser:self.currentUser WithPredicateString:[NSPredicate predicateWithFormat:@"username == %@", self.currentUser.username]];
                loginVerified = YES;
                [self launchHomeView];
            }
        }
        
        /*
         This might need to be added if we are failing to add user due to some other reason aside from the username being taken
        UIAlertView *failedToAddUser = [[UIAlertView alloc] initWithTitle:@"Failed to add user at this time" message:@"We were unable to create a user with the information provided. Please try again." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [failedToAddUser show];
        */
        
    } onError:^(NSError *error) {
        
        DLog(@"%@", error);
        UIAlertView *failedToConnectToServer = [[UIAlertView alloc] initWithTitle:@"Failed to connect to the Simply Selfcare Server" message:@"We apologize, please try again soon." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [failedToConnectToServer show];
    }];
    
    [[[SSCMKNetworkSingleton sharedInstance] nwe] enqueueOperation:op];
}

#pragma mark UIView And Keyboard Handling
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    if (textField == dob_day || textField == dob_month) {
        NSUInteger newLength = [textField.text length] + [string length] - range.length;
        return (newLength > 2) ? NO : YES;
    }
    else if (textField == dob_year) {
        NSUInteger newLength = [textField.text length] + [string length] - range.length;
        return (newLength > 4) ? NO : YES;
    }
    else {
        return YES;
    }
}

#pragma mark UITextViewDelegate
- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    NSInteger nextTag = textField.tag + 1;
    // Try to find next responder
    if (isSigningIn && nextTag >= 2) {
        [textField resignFirstResponder];
    }
    else {
    UIResponder* nextResponder = [textField.superview viewWithTag:nextTag];
    if (nextResponder) {
        [nextResponder becomeFirstResponder];
    } else {
        [textField resignFirstResponder];
    }
    }
    return NO;
}


#pragma mark Utility Methods

- (NSDate*)birthDateFromLoginFields {
    NSString *dateOfBirthTemp = [NSString stringWithFormat:@"%@/%@/%@",dob_month.text,dob_day.text,dob_year.text];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"MM/dd/yyyy"];
    NSDate *dateOfBirth = [dateFormatter dateFromString:dateOfBirthTemp];
    return dateOfBirth;
}

#pragma mark View Animations 

-(void)animateLoginFormButtons {
    if (isCreatingNewUser) {
        //Rename Sign in button, and move to make room for new user fields
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationDelegate:self];
        [UIView setAnimationDidStopSelector:@selector(secondHalfOfLoginFormAnimation:finished:context:)];
        [UIView setAnimationCurve:UIViewAnimationCurveLinear];
        [UIView setAnimationDuration:.5];
        signIn.frame = CGRectMake(signIn.frame.origin.x, signIn.frame.origin.y + 115, signIn.frame.size.width, signIn.frame.size.height);
        signIn.titleLabel.text = @"Cancel";
        createNew.frame = CGRectMake(createNew.frame.origin.x, createNew.frame.origin.y + 115, createNew.frame.size.width, createNew.frame.size.height);
        [UIView commitAnimations];
    }
    else {
        //cancel button hit, move everything back
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationDelegate:self];
        [UIView setAnimationDidStopSelector:@selector(secondHalfOfLoginFormAnimation:finished:context:)];
        [UIView setAnimationCurve:UIViewAnimationCurveEaseIn];
        [UIView setAnimationDuration:.5];
        
        [self.email setAlpha:0];
        [self.gender setAlpha:0];
        [self.dob_day setAlpha:0];
        [self.dob_month setAlpha:0];
        [self.dob_year setAlpha:0];
        [self.dob_slash1 setAlpha:0];
        [self.dob_slash2 setAlpha:0];
        
        [UIView commitAnimations];
    }
    
}

-(void)secondHalfOfLoginFormAnimation:(NSString *)animationID finished:(NSNumber *)finished context:(void *)context {
    if (isCreatingNewUser) {
        //move fields into create new user mode
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationCurve:UIViewAnimationCurveEaseOut];
        [UIView setAnimationDuration:.5];
        
        [self.email setAlpha:1];
        [self.gender setAlpha:1];
        [self.dob_day setAlpha:1];
        [self.dob_month setAlpha:1];
        [self.dob_year setAlpha:1];
        [self.dob_slash1 setAlpha:1];
        [self.dob_slash2 setAlpha:1];
        
        [UIView commitAnimations];
    }
    else {
        //Move everything back into sign in mode
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationCurve:UIViewAnimationCurveLinear];
        [UIView setAnimationDuration:.5];
        signIn.frame = CGRectMake(signIn.frame.origin.x, signIn.frame.origin.y - 115, signIn.frame.size.width, signIn.frame.size.height);
        signIn.titleLabel.text = @"Sign In";
        createNew.frame = CGRectMake(createNew.frame.origin.x, createNew.frame.origin.y - 115, createNew.frame.size.width, createNew.frame.size.height);
        [UIView commitAnimations];
    }
    
}

#pragma mark Keyboard Notification Handling
// Handles hiding the keyboard when the view is tapped
-(void)hideKeyboard
{
    [[self view] endEditing:TRUE];
}

- (void)keyboardWillHide:(NSNotification *)n
{
    NSDictionary* userInfo = [n userInfo];
    
    // get the size of the keyboard
    CGSize keyboardSize = [[userInfo objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;
    
    
    // resize the scrollview
    CGRect viewFrame = self.loginFormScrollView.frame;
    // I'm also subtracting a constant kTabBarHeight because my UIScrollView was offset by the UITabBar so really only the portion of the keyboard that is leftover pass the UITabBar is obscuring my UIScrollView.
    viewFrame.size.height += keyboardSize.height;
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationBeginsFromCurrentState:YES];
    // The kKeyboardAnimationDuration I am using is 0.3
    [UIView setAnimationDuration:.3];
    [self.loginFormScrollView setFrame:viewFrame];
    [UIView commitAnimations];
    
    keyboardIsShown = NO;
}

- (void)keyboardWillShow:(NSNotification *)n
{
    // This is an ivar I'm using to ensure that we do not do the frame size adjustment on the UIScrollView if the keyboard is already shown.  This can happen if the user, after fixing editing a UITextField, scrolls the resized UIScrollView to another UITextField and attempts to edit the next UITextField.  If we were to resize the UIScrollView again, it would be disastrous.  NOTE: The keyboard notification will fire even when the keyboard is already shown.
    if (keyboardIsShown) {
        return;
    }
    
    NSDictionary* userInfo = [n userInfo];
    
    // get the size of the keyboard
    CGSize keyboardSize = [[userInfo objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;
    
    // resize the noteView
    CGRect viewFrame = self.loginFormScrollView.frame;
    // I'm also subtracting a constant kTabBarHeight because my UIScrollView was offset by the UITabBar so really only the portion of the keyboard that is leftover pass the UITabBar is obscuring my UIScrollView.
    viewFrame.size.height -= keyboardSize.height;
    
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationBeginsFromCurrentState:YES];
    // The kKeyboardAnimationDuration I am using is 0.3
    [UIView setAnimationDuration:.3];
    [self.loginFormScrollView setFrame:viewFrame];
    [UIView commitAnimations];
    
    keyboardIsShown = YES;
}
@end
