//
//  SSCUser.m
//  SimplySelfCare
//
//  Created by Michael Place on 8/24/12.
//  Copyright (c) 2012 scrapsfromthetable. All rights reserved.
//

#import "SSCUser.h"
#import "SSCMKNetworkSingleton.h"

@implementation SSCUser
@synthesize username, password, dob, isPreg, isLac, gender, email;
@synthesize currentConditions, currentMedications, medicineCabinet;

- (SSCUser*) initUserWith:(NSString*)username_ withPassword:(NSString*)password_ withEmail:(NSString*)email_ withGender:(NSNumber*)gender_ withDOB:(NSDate*)dob_
          withIsPregnant:(NSNumber*)isPreg_ withIsLactating:(NSNumber*)isLac_ withCurrentMedications:(NSArray*)currentMedications_ withCurrentConditions:(NSArray*)currentConditions_ withMedicineCabinet:(NSArray*)medicineCabinet_ {
    if (self = [super init]) {
    self.username = username_;
    self.password = password_;
    self.email = email_;
    self.Gender = gender_;
    self.dob = dob_;
    self.isPreg = isPreg_;
    self.isLac = isLac_;
    self.currentMedications = currentMedications_;
    self.currentConditions = currentConditions_;
    self.medicineCabinet = medicineCabinet_;
    }
    return self;
}

- (SSCUser *)initUserFromManagedObject:(NSManagedObject*)mo {
    if (self = [super init]) {
        self.username = [mo valueForKey:@"username"];
        self.password = [mo valueForKey:@"password"];
        self.email = [mo valueForKey:@"email"];
        self.Gender = (Male == (NSInteger)[mo valueForKey:@"gender"] ? [NSNumber numberWithInt:0] : [NSNumber numberWithInt:1]);
        self.dob = [mo valueForKey:@"dob"];
        self.isPreg = (Pregnant == (NSInteger)[mo valueForKey:@"ispregnant"] ? [NSNumber numberWithInt:0] : [NSNumber numberWithInt:1]);;
        self.isLac = (Lactating == (NSInteger)[mo valueForKey:@"islactating"] ? [NSNumber numberWithInt:0] : [NSNumber numberWithInt:1]);;
        self.currentMedications = [SSCUser arrayFromStringOfConditionsOrMedications:(NSString*)[mo valueForKey:@"currentmedications"]];
        self.currentConditions = [SSCUser arrayFromStringOfConditionsOrMedications:(NSString*)[mo valueForKey:@"currentconditions"]];
        self.medicineCabinet = @[];
    }
    return self;
}


- (SSCUser*)init {
    if (self = [super init]) {
    self.username = @"";
    self.password = @"";
    self.email = @"";
    self.Gender = [NSNumber numberWithInt:0];
    self.dob = [NSDate date];
    self.isPreg = [NSNumber numberWithInt:0];
    self.isLac = [NSNumber numberWithInt:0];
    self.currentMedications = @[];
    self.currentConditions = @[];
    self.medicineCabinet = @[];
    }
    return self;
}

- (NSDictionary *)dictionaryObjectForUser:(SSCUser*)user {
    //Get password from keychain in future
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"MM/dd/yyyy"];
    NSString *dateOfBirthString = [dateFormatter stringFromDate:user.dob];
    
    NSArray *dictionaryKeys = [[NSArray alloc] initWithObjects:@"username", @"password", @"email", @"gender", @"dob", @"ispregnant", @"islactating", @"currentmedications", @"currentconditions", @"medicinecabinet", nil];
    NSArray *dictionaryValues = [[NSArray alloc] initWithObjects:user.username, user.password, user.email, user.gender, dateOfBirthString, user.isPreg, user.isLac, user.currentMedications, user.currentConditions, user.medicineCabinet, nil];
    
    NSDictionary *dictonaryForUser = [[NSDictionary alloc] initWithObjects:dictionaryValues forKeys:dictionaryKeys];
    
    return dictonaryForUser;
}

- (NSData*)jsonRepresentationForUserDictionary:(NSDictionary*)userDict {
    NSError *error;
    NSData* jsonUserData = [NSJSONSerialization dataWithJSONObject:userDict
                                                       options:NSJSONWritingPrettyPrinted error:&error];
        
    return jsonUserData;
    
}


+ (NSDate*)birthDateFromString:(NSString*)dateOfBirth {
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"MM/dd/yyyy"];
    NSDate *date = [dateFormatter dateFromString:dateOfBirth];
    return date;
}

+ (NSNumber*)genderNumberFromString:(NSString*)genderString {
    NSNumber *gender;
    if ([genderString isEqualToString:@"Male"]) {
        gender = [NSNumber numberWithInt:Male];
    }
    else {
        gender = [NSNumber numberWithInt:Female];
    }
    return gender;
}

+ (NSArray*)arrayFromStringOfConditionsOrMedications:(NSString*)a {
    NSArray *returned = [a componentsSeparatedByString:@","];
    return returned;
}

+ (NSString*)stringFromArrayOfConditionsOrMedications:(NSArray*)a {
    NSString *conditionOrMedString = [a componentsJoinedByString:@","];
    return conditionOrMedString;
}

+ (void)manageLocalUserStoreWithUser:(SSCUser*)user {
    NSManagedObjectContext *moc = [[SSCDiagnosisStore sharedInstance] managedObjectContext];
    NSManagedObject *userEntry = [NSEntityDescription
                                  insertNewObjectForEntityForName:@"User"
                                  inManagedObjectContext:moc];
    [userEntry setValue:user.username forKey:@"username"];
    [userEntry setValue:user.email forKey:@"email"];
    [userEntry setValue:user.dob forKey:@"dob"];
    [userEntry setValue:user.gender forKey:@"gender"];
    [userEntry setValue:[self stringFromArrayOfConditionsOrMedications:user.currentConditions] forKey:@"currentconditions"];
    [userEntry setValue:[self stringFromArrayOfConditionsOrMedications:user.currentMedications] forKey:@"currentmedications"];
    [userEntry setValue:user.password forKey:@"password"];
    
    NSError *error;
    if (![moc save:&error]) {
        NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
    }
}

+ (NSArray *)fetchAllUsers {
    NSManagedObjectContext *moc = [[SSCDiagnosisStore sharedInstance] managedObjectContext];
    NSEntityDescription *entity = [NSEntityDescription
                                   entityForName:@"User" inManagedObjectContext:moc];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entity];
    return [moc executeFetchRequest:request error:nil];
}

+ (SSCUser*)fetchUserWithUsername:(NSString*)username {
    NSManagedObjectContext *moc = [[SSCDiagnosisStore sharedInstance] managedObjectContext];
    NSEntityDescription *entity = [NSEntityDescription
                                   entityForName:@"User" inManagedObjectContext:moc];
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"username == %@", username];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setPredicate:pred];
    [request setEntity:entity];
    NSArray *users = [moc executeFetchRequest:request error:nil];
    SSCUser *user = [[SSCUser alloc] initUserFromManagedObject:[users objectAtIndex:0]];
    return user;
}

+ (void)fetchUser:(SSCUser*)user WithPredicateString:(NSPredicate*)predicate {
    // Retrieve user data from core data
    NSManagedObjectContext *moc = [[SSCDiagnosisStore sharedInstance] managedObjectContext];
    NSEntityDescription *entityDescription = [NSEntityDescription
                                              entityForName:@"User" inManagedObjectContext:moc];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entityDescription];
    
    // Set example predicate and sort orderings...
    
    [request setPredicate:predicate];
    
    NSError *error;
    NSArray *array = [moc executeFetchRequest:request error:&error];
    if (array == nil)
    {
        // Deal with error...
        NSLog(@"ERROR WRITING TO USER STORE");
    }
    else {
        if ([array count] == 0) {
            NSLog(@"No entries were found for user with predicate string %@", [predicate description]);
            [self manageLocalUserStoreWithUser:user];
        }
        else {
            NSManagedObject *u = [array objectAtIndex:0];
            [u setValue:user.username forKey:@"username"];
            [u setValue:user.email forKey:@"email"];
            [u setValue:user.dob forKey:@"dob"];
            [u setValue:user.gender forKey:@"gender"];
            [u setValue:[self stringFromArrayOfConditionsOrMedications:user.currentConditions] forKey:@"currentconditions"];
            [u setValue:[self stringFromArrayOfConditionsOrMedications:user.currentMedications] forKey:@"currentmedications"];
            [u setValue:user.password forKey:@"password"];
            
            NSError *error;
            if (![moc save:&error]) {
                NSLog(@"Whoops, couldn't save: %@", [error localizedDescription]);
                //UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"User Successfully Updated!" message:[NSString stringWithFormat:@"username: %@, email: %@, gender: %@, dob: %@", [u valueForKey:@"username"], [u valueForKey:@"email"], [u valueForKey:@"gender"], [u valueForKey:@"dob"]] delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil, nil];
                //[alert show];
            }
        }
        
        
    }
}

+ (void)applyProfileUserChangesRemotelyWithUser:(SSCUser*)user {
    // send updated user data to server
    NSDictionary *userDict = [user dictionaryObjectForUser:user];
    NSData *jsonUserData = [user jsonRepresentationForUserDictionary:userDict];
    NSString* stringData = [[NSString alloc] initWithData:jsonUserData
                                                 encoding:NSUTF8StringEncoding];
    NSMutableDictionary *jsonMutableDictionary = [[NSMutableDictionary alloc]initWithDictionary:@{@"ModifyUser" : stringData}];
    
    MKNetworkOperation *op = [[[SSCMKNetworkSingleton sharedInstance] nwe] operationWithPath:kSSCAuth
                                                                                      params:jsonMutableDictionary
                                                                                  httpMethod:@"POST"];
    [op onCompletion:^(MKNetworkOperation *operation) {
        
        NSDictionary *jsonResponseDictionary = [operation responseJSON];
        if (nil == jsonResponseDictionary) {
            NSLog(@"RESPONSE FROM SERVER IS NIL!!");
        }
        if ([[jsonResponseDictionary allKeys] containsObject:@"requestStatus"] && ![[jsonResponseDictionary objectForKey:@"requestStatus"] isEqualToNumber:[NSNumber numberWithInt:0]]) {
            UIAlertView *usernameAlreadyExistsInDatabase = [[UIAlertView alloc] initWithTitle:@"Failed to update user info remotely" message:@"We were unable to update your user information on our servers at this time. Please try again." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [usernameAlreadyExistsInDatabase show];
        }
        else {
            
            //User has been added so current user is good they way it was initialized
            NSLog(@"User was updated succesfully");
        }
    }onError:^(NSError *error) {
        
        DLog(@"%@", error);
        UIAlertView *failedToConnectToServer = [[UIAlertView alloc] initWithTitle:@"Failed to connect to the Simply Selfcare Server" message:@"We apologize, please try again soon." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [failedToConnectToServer show];
    }];
    [[[SSCMKNetworkSingleton sharedInstance] nwe] enqueueOperation:op];
    
}


@end
