//
//  SSCViableMedViewController.m
//  SimplySelfCare
//
//  Created by Michael Place on 3/16/13.
//  Copyright (c) 2013 scrapsfromthetable. All rights reserved.
//

#import "SSCViableMedViewController.h"
#import "SSCBaseViewController.h"

@interface SSCViableMedViewController ()

@end

@implementation SSCViableMedViewController
@synthesize currentUser = _currentUser;
@synthesize viableMeds = _viableMeds;
@synthesize drugSaveStatusArray = _drugSaveStatusArray;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    _currentRowSelected = 0;
    [self.viableDrugTableView reloadData];
    [self setUpDrugSaveStatusArray];
    [self updateDrugInfoViewWithIndex:0];
    // Do any additional setup after loading the view from its nib.
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewDidUnload {
    [self setDrugNameLabel:nil];
    [self setInformationTextView:nil];
    [self setViableDrugTableView:nil];
    [self setSaveButton:nil];
    [super viewDidUnload];
}

- (void)setUpDrugSaveStatusArray {
    self.drugSaveStatusArray = [[NSMutableArray alloc] init];
    for (int i = 0; i < [self.viableMeds count]; i++) {
        [self.drugSaveStatusArray addObject:[NSNumber numberWithBool:NO]];
    }
}

- (BOOL)currentUserIsAlreadyTakingSelectedDrug {
    return [self.currentUser.currentMedications containsObject:[self.viableMeds objectAtIndex:_currentRowSelected]];
}

- (void)updateSaveStatusArrayWithIndex:(int)index {
    if (![self currentUserIsAlreadyTakingSelectedDrug]) {
        [self.drugSaveStatusArray setObject:([self.drugSaveStatusArray objectAtIndex:index] == [NSNumber numberWithBool:NO]) ? [NSNumber numberWithBool:YES] : [NSNumber numberWithBool:NO] atIndexedSubscript:index];
    }
    else {
        [self.drugSaveStatusArray setObject:[NSNumber numberWithBool:NO] atIndexedSubscript:index];
    }
}

- (void)setSaveButtonText {
    if (![self currentUserIsAlreadyTakingSelectedDrug]) {
        [self.saveButton setTitle:([[self.drugSaveStatusArray objectAtIndex:_currentRowSelected] boolValue]) ? @"Remove" : @"Save" forState:UIControlStateNormal];
    }
    else {
        [self.saveButton setTitle:@"Taking.." forState:UIControlStateNormal];
        [self.saveButton setEnabled:NO];
    }
}

- (void)updateDrugInfoViewWithIndex:(int)index {
    if ([self.viableMeds count] > 0 && !([(NSString*)[self.viableMeds objectAtIndex:0] isEqual: @""])) {
        NSString *drugName = [[self.viableMeds objectAtIndex:index] stringByReplacingCharactersInRange:NSMakeRange(0,1) withString:[[[self.viableMeds objectAtIndex:index] substringToIndex:1] uppercaseString]];
        [self.drugNameLabel setText:drugName];
        [self setSaveButtonText];
    }
    else {
        [self.drugNameLabel setText:@"No Medications Found"];
        [self.saveButton setEnabled:NO];
    }
}

- (IBAction)saveDrugButtonWasClicked:(UIButton *)sender {
    NSNumber *setSaved = ([self.drugSaveStatusArray objectAtIndex:_currentRowSelected] == [NSNumber numberWithBool:YES]) ? [NSNumber numberWithBool:NO] : [NSNumber numberWithBool:YES];
    [self.drugSaveStatusArray setObject:setSaved atIndexedSubscript:_currentRowSelected];
    [self setSaveButtonText];
    
}

- (void)updateUser {
    NSMutableArray *currentMedication = [[NSMutableArray alloc] init];
    for (int i = 0; i < [self.drugSaveStatusArray count]; i++) {
        if ([[self.drugSaveStatusArray objectAtIndex:i] isEqualToNumber:[NSNumber numberWithBool:YES]]) {
            [currentMedication addObject:[[self.viableMeds objectAtIndex:i] lowercaseString]];
        }
    }
    if ([currentMedication count] > 0) {
        self.currentUser.currentMedications = [currentMedication arrayByAddingObjectsFromArray:self.currentUser.currentMedications];
        [SSCUser fetchUser:self.currentUser WithPredicateString:[NSPredicate predicateWithFormat:@"username == %@",self.currentUser.username]];
        [SSCUser applyProfileUserChangesRemotelyWithUser:self.currentUser];
    }
}

- (IBAction)doneButtonWasClicked:(UIBarButtonItem *)sender {
    [self updateUser];
    [(SSCBaseViewController*)[self.navigationController.viewControllers objectAtIndex:0] invalidateUserForUserWithUsername:self.currentUser.username];
    [self.navigationController popToRootViewControllerAnimated:YES];
}

- (IBAction)backButtonWasClicked:(UIBarButtonItem *)sender {
    [self.navigationController popViewControllerAnimated:YES];
}

- (IBAction)cancelButtonWasClicked:(UIBarButtonItem *)sender {
    [self.navigationController popToRootViewControllerAnimated:YES];
}

#pragma -TableViewDelegateMethods

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
    
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"Cell"];
    }
    
    // Add elements to the cell
    [cell.textLabel setText:[self.viableMeds objectAtIndex:[indexPath row]]];
    
    return cell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.viableMeds count];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    ([[tableView cellForRowAtIndexPath:indexPath] isSelected]) ? nil : [[tableView cellForRowAtIndexPath:indexPath] setSelected:YES];
    _currentRowSelected = [indexPath row];
    [self updateDrugInfoViewWithIndex:[indexPath row]];
}
@end
